/*
 *  Copyright (C) 2000, 2001 Robert Russell <rdr@cs.unh.edu>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

"divide_error",				/*  0 */
"debug",					/*  1 */
"nmi",						/*  2 */
"int3",						/*  3 */
"overflow",					/*  4 */
"bounds",					/*  5 */
"invalid_op",				/*  6 */
"device_not_available",		/*  7 */
"double_fault",				/*  8 */
"coprocessor_seg_overrun",	/*  9 */
"invalid_TSS",				/* 10 */
"segment_not_present",		/* 11 */
"stack_segment",			/* 12 */
"general_protection",		/* 13 */
"page_fault",				/* 14 */
"spurious_interrupt_bug",	/* 15 */
"coprocessor_error",		/* 16 */
"alignment_check",			/* 17 */
"machine_check",			/* 18 */
"simd_coprocessor_error",	/* 19 */
"unknown trap",				/* 20 */
/* vim: ts=4
 */
