/*
 *  Copyright (C) 2000, 2001 Robert Russell <rdr@cs.unh.edu>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

"local perfctr",	/* 0xee */
"local timer",		/* 0xef */
"thermal APIC",		/* 0xf0 */
"reserved f1",		/* 0xf1 */
"reserved f2",		/* 0xf2 */
"reserved f3",		/* 0xf3 */
"reserved f4",		/* 0xf4 */
"reserved f5",		/* 0xf5 */
"reserved f6",		/* 0xf6 */
"reserved f7",		/* 0xf7 */
"reserved f8",		/* 0xf8 */
"reserved f9",		/* 0xf9 */
"reserved fa",		/* 0xfa */
"call function",	/* 0xfb */
"reschedule",		/* 0xfc */
"invalidate TLB",	/* 0xfd */
"error APIC",		/* 0xfe */
"spurious APIC",	/* 0xff */
